"""Constants used in package

Attributes:
  PKG_INDICES (dict): Key is language abbreviation, such as ``en``, ``es``. Value is the name of pre-trained package 
"""

PKG_INDICES = {
  "en": "en_core_web_trf",
  "es": "es_dep_news_trf",
  "de": "de_dep_news_trf",
  "fr": "fr_dep_news_trf",
}

