"""Define regular expression usage of ``spacy.matcher.Matcher`` for ``English``
"""

# VERB Knowledge pattern
EN_VERB = [
  [{"POS": "VERB"}],
  [{"TAG": "VB"}],
  [{"TAG": "VBZ"}],
  [{"TAG": "VBN"}],
  [{"TAG": "VBP"}],
  [{"TAG": "VBG"}],
  [{"TAG": "VBD"}],
]

